//----------------
/*[[
Name := GridMaster_03
Notes:= This is a pure, 100% REACTIVE
strategy, with no attempt at all to predict, anticipate or define a statistical
edge. It limits grid size by keeping edges trimmed. This version (03) increases the
profit target whenever an edge is trimmed. The ProfitIncrease define is used to
indicate how much to increase the profit target. The MaxProfitExit define is used
to limit the profit target to the defined value.
Lots := 1.0
Stop Loss := 0
Take Profit := 0
Trailing Stop := 0
]]*/


// Defines;
// Open new trades if TotalProfit less than this value
// Stop opening new trades and start closing them all if TotalProfit greater than this value
// Dollar amount to increase profit target each time an edge is trimmed
// Maximum profit target
// Number of TotalTrades allowed
// Number of trades by which buys can exceed sells and vice versa

// Presets;

#include <stdlib.mqh>
//+------------------------------------------------------------------+
//| GridMaster_03 |
//| |
//| |
//| Converted by Mql2Mq4 v1.1 |
//| http://yousky.free.fr |
//| Copyright  2006, Yousky Soft |
//+------------------------------------------------------------------+

#property copyright ""
#property link ""


//+------------------------------------------------------------------+
//| Common External variables |
//+------------------------------------------------------------------+
extern double Lots = 0.1;
extern double StopLoss = 0.00;
extern double TakeProfit = 0.00;
extern double TrailingStop = 0.00;

//+------------------------------------------------------------------+
//| External variables |
//+------------------------------------------------------------------+
extern double ProfitEntry = 10;
extern double ProfitExit = 100;
extern double ProfitIncrease = 50;
extern double MaxProfitExit = 50;
extern double MaxTrades = 50;
extern double BalanceDelta = 10;
extern double BucksPerPip = 0.1;
extern double dFreeMargin = 2000;
extern double Slippage = 4;
extern double  ratio          = 0.00001;

//+------------------------------------------------------------------+
//| Special Convertion Functions |
//+------------------------------------------------------------------+

int LastTradeTime;

int MOrderSend( string symbol, int cmd, double volume, double price, int slippage, double stoploss, double takeprofit, string comment="", int magic=0, datetime expiration=0, color arrow_color=CLR_NONE)
{
LastTradeTime = CurTime();
price = MathRound(price*10000)/10000;
stoploss = MathRound(stoploss*10000)/10000;
takeprofit = MathRound(takeprofit*10000)/10000;
return ( OrderSend( symbol, cmd, volume, price, slippage, stoploss, takeprofit, comment, magic, expiration, arrow_color ) );
}

int OrderValueTicket(int index)
{
OrderSelect(index, SELECT_BY_POS);
return(OrderTicket());
}

int OrderValueType(int index)
{
OrderSelect(index, SELECT_BY_POS);
return(OrderType());
}

double OrderValueLots(int index)
{
OrderSelect(index, SELECT_BY_POS);
return(OrderLots());
}

double OrderValueOpenPrice(int index)
{
OrderSelect(index, SELECT_BY_POS);
return(OrderOpenPrice());
}

string OrderValueSymbol(int index)
{
OrderSelect(index, SELECT_BY_POS);
return(OrderSymbol());
}

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//+------------------------------------------------------------------+
//| Local variables |
//+------------------------------------------------------------------+
int i = 0;
double ProfitTarget = 0;
double NumBuys = 0;
double NumSells = 0;
double CumulativeBuyRate = 0;
double CumulativeSellRate = 0;
double BuyPL = 0;
double SellPL = 0;
double CloseAl1 = 0;
double OverallPL = 0;
double ExitRate = 0;
double LastTradeWasBuy = 0;
double LastTradeWasSell = 0;
double HighestSell = 0;
double LowestBuy = 0;
double HighestBuy = 0;
double LowestSell = 0;
double BuyNewLevel = 0;
double SellNewLevel = 0;
double CloseAll = 0;
double TimeClock = 0;
double TimeMin = 0;
double LargestMarginUsed = 0;
double LargestFloatingLoss = 0;
double CloseSwitch = 0;
double CloseSellFirst = 0;
double CloseBuyFirst = 0;
double LowestFreeMargin = 0;

if( CurTime() - LastTradeTime < 10 ) return(0);
if( AccountMargin() > LargestMarginUsed ) LargestMarginUsed = AccountMargin();
if( AccountProfit() < LargestFloatingLoss ) LargestFloatingLoss = AccountProfit();
if( LowestFreeMargin == 0 ) LowestFreeMargin = AccountFreeMargin();
if( AccountFreeMargin() < LowestFreeMargin ) LowestFreeMargin = AccountFreeMargin();


NumBuys = 0;
NumSells = 0;
CumulativeBuyRate = 0.0;
CumulativeSellRate = 0.0;
OverallPL = 0;

for(i =1;i <=OrdersTotal();i ++)
{
if( OrderValueSymbol(i) == Symbol() && OrderValueType(i) == OP_BUY )
{
NumBuys++;
CumulativeBuyRate += OrderValueOpenPrice(i);
}

if( OrderValueSymbol(i) == Symbol() && OrderValueType(i) == OP_SELL )
{
NumSells++;
CumulativeSellRate += OrderValueOpenPrice(i);
}
}

if( (NumBuys > NumSells && NumSells > 0) )
{
ExitRate = CumulativeBuyRate/NumBuys;
while( (OverallPL < ProfitTarget)
) {
BuyPL = ((BucksPerPip*NumBuys)/Point)*(ExitRate - CumulativeBuyRate/NumBuys);
SellPL = ((BucksPerPip*NumSells)/Point)*(CumulativeSellRate/NumSells - ExitRate);
ExitRate += Point;
OverallPL = BuyPL + SellPL;
}

if( ProfitTarget < ProfitExit ) ProfitTarget = ProfitExit;
}
else
if( (NumSells > NumBuys && NumBuys > 0) )
{
ExitRate = CumulativeSellRate/NumSells;
while( (OverallPL < ProfitTarget)
) {
BuyPL = ((BucksPerPip*NumBuys)/Point)*(ExitRate - CumulativeBuyRate/NumBuys);
SellPL = ((BucksPerPip*NumSells)/Point)*(CumulativeSellRate/NumSells - ExitRate);
ExitRate -= Point;
OverallPL = BuyPL + SellPL;
}

if( ProfitTarget < ProfitExit ) ProfitTarget = ProfitExit;
}


if( CloseBuyFirst == 1 ||
CloseSellFirst == 1 )
{
if( NumBuys == 0 ) CloseBuyFirst = 0;
if( NumSells == 0 ) CloseSellFirst = 0;
}

// if Profit is positive, close all open positions;

if( AccountProfit() > ProfitTarget && CloseSwitch == 0 )
{
CloseAll = 1;
CloseSwitch = 1;

if( NumBuys > NumSells ) CloseSellFirst = 1;
if( NumSells > NumBuys ) CloseBuyFirst = 1;
}

if( OrdersTotal() == 0 )
{
CloseAll = 0;
CloseSwitch = 0;
ProfitTarget = ProfitExit;
}

if( CloseAl1 == 1 )
{
for(i =1;i <=OrdersTotal();i ++)
{
if( OrderValueSymbol(i) == Symbol() && OrderValueType(i) == OP_BUY && (CloseBuyFirst == 1) ||
(CloseBuyFirst == 0 && CloseSellFirst == 0) )
{
if( Bid-OrderValueOpenPrice(i) < 0 )
{
OrderClose(OrderValueTicket(i),OrderValueLots(i),Bid,Slippage,LawnGreen);
return(0);
}

if( Bid-OrderValueOpenPrice(i) > 0 )
{
OrderClose(OrderValueTicket(i),OrderValueLots(i),Bid,Slippage,Gold);
return(0);
}
}

if( OrderValueSymbol(i) == Symbol() && OrderValueType(i) == OP_SELL && (CloseSellFirst == 1) ||
(CloseBuyFirst == 0 && CloseSellFirst == 0) )
{
if( OrderValueOpenPrice(i)-Ask < 0 )
{
OrderClose(OrderValueTicket(i),OrderValueLots(i),Ask,Slippage,HotPink);
return(0);
}

if( OrderValueOpenPrice(i)-Ask > 0 )
{
OrderClose(OrderValueTicket(i),OrderValueLots(i),Ask,Slippage,Gold);
return(0);
}
}
}
}


if( (CloseAll == 0) ) // If closing switch is set to 0, we can open new orders and adjust TPs/SLs
{
// Open First Trades;

if( NumBuys >= 0 || NumSells >= 0 )
{
if( NumBuys == 0 && AccountFreeMargin() > dFreeMargin )
{
MOrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,0,"",16384,0,Blue);
LastTradeWasBuy = 1;
LastTradeWasSell = 0;
return(0);
}

if( NumSells == 0 && AccountFreeMargin() > dFreeMargin )
{
MOrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,0,0,"",16384,0,Red);
LastTradeWasSell = 1;
LastTradeWasBuy = 0;
return(0);
}

////////////////////////////////////////////////////////
// Determine Highest Buy and Lowest Sell;
////////////////////////////////////////////////////////
LowestBuy = 1000;
HighestSell = 0;
HighestBuy = 0;
LowestSell = 1000;

for(i =1;i <=OrdersTotal();i ++)
{
if( OrderValueSymbol(i) == Symbol() && OrderValueType(i) == OP_BUY && OrderValueOpenPrice(i) < LowestBuy ) LowestBuy = OrderValueOpenPrice(i);
if( OrderValueSymbol(i) == Symbol() && OrderValueType(i) == OP_SELL && OrderValueOpenPrice(i) > HighestSell ) HighestSell = OrderValueOpenPrice(i);
if( OrderValueSymbol(i) == Symbol() && OrderValueType(i) == OP_BUY && OrderValueOpenPrice(i) > HighestBuy ) HighestBuy = OrderValueOpenPrice(i);
if( OrderValueSymbol(i) == Symbol() && OrderValueType(i) == OP_SELL && OrderValueOpenPrice(i) < LowestSell ) LowestSell = OrderValueOpenPrice(i);
}

/////////////////////////////////////////
// If TotalTrades > MaxTrades, Close Highest Buy If Last Trade Was A Sell
// Or Close Lowest Sell If Last Trade Was A Buy
//////////////////////////////////////////
for(i =1;i <=OrdersTotal();i ++)
{
if( LastTradeWasSell == 1 &&
OrdersTotal() > MaxTrades &&
OrderValueSymbol(i) == Symbol() &&
OrderValueType(i) == OP_BUY &&
OrderValueOpenPrice(i) == HighestBuy )
{
ProfitTarget += ProfitIncrease;
if( ProfitTarget > MaxProfitExit ) ProfitTarget = MaxProfitExit;
OrderClose(OrderValueTicket(i),OrderValueLots(i),Bid,Slippage,Violet);
return(0);
}

if( LastTradeWasBuy == 1 &&
OrdersTotal() > MaxTrades &&
OrderValueSymbol(i) == Symbol() &&
OrderValueType(i) == OP_SELL &&
OrderValueOpenPrice(i) == LowestSell )
{
ProfitTarget += ProfitIncrease;
if( ProfitTarget > MaxProfitExit ) ProfitTarget = MaxProfitExit;
OrderClose(OrderValueTicket(i),OrderValueLots(i),Ask,Slippage,Violet);
return(0);
}
}


// BuyNewLevel, SellNewLevel;

BuyNewLevel = 0;
SellNewLevel = 0;

for(i =1;i <=OrdersTotal();i ++)
{
if( ((OrderValueSymbol(i) == Symbol()) && (OrderValueType(i) == OP_BUY) && (OrderValueOpenPrice(i) == Ask)) ) BuyNewLevel = 1;
if( ((OrderValueSymbol(i) == Symbol()) && (OrderValueType(i) == OP_SELL) && (OrderValueOpenPrice(i) == Bid)) ) SellNewLevel = 1;
}

////////////////////////////////////////////////////////
// Open additional trades on a grid
////////////////////////////////////////////////////////
if( Ask > LowestBuy && BuyNewLevel == 0 &&
AccountProfit() < ProfitEntry &&
NumBuys <= NumSells+BalanceDelta )
{
if( AccountFreeMargin() < dFreeMargin ) return(0);
LastTradeWasBuy = 1;
LastTradeWasSell = 0;
MOrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,0,0,"",16384,0,Blue);
return(0);
}

if( Bid < HighestSell && SellNewLevel == 0 &&
AccountProfit() < ProfitEntry &&
NumSells <= NumBuys+BalanceDelta )
{
if( AccountFreeMargin() < dFreeMargin ) return(0);
LastTradeWasSell = 1;
LastTradeWasBuy = 0;
MOrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,0,0,"",16384,0,Red);
return(0);
}
}
}
return(0); return(0);
}
//----------------